"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const babel = __importStar(require("@babel/core"));
const bluebird_1 = require("bluebird");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
class BabelCompiler {
    constructor(id, logger, compiler, options) {
        this.id = id;
        this.logger = logger;
        this.compiler = compiler;
        this.options = options;
        this.distDir = 'dist';
        this.distGlobPatterns = [`${this.distDir}/**`];
        this.shouldCopyNonSupportedFiles = true;
        this.artifactName = 'dist';
    }
    /**
     * compile one file on the workspace
     */
    transpileFile(fileContent, options) {
        const supportedExtensions = ['.ts', '.tsx', '.js', '.jsx'];
        const fileExtension = path_1.default.extname(options.filePath);
        if (!supportedExtensions.includes(fileExtension) || options.filePath.endsWith('.d.ts')) {
            return null; // file is not supported
        }
        const transformOptions = this.options.babelTransformOptions || {};
        // the `sourceRoot` and `sourceFileName` are manually set because the dists are written into the
        // node_modules dir, so the debugger needs to know where to find the source.
        transformOptions.sourceRoot = options.componentDir;
        transformOptions.sourceFileName = options.filePath;
        transformOptions.filename = options.filePath;
        const result = babel.transformSync(fileContent, this.options.babelTransformOptions);
        if (!result) {
            throw new Error(`babel returns no result`);
        }
        const code = result.code || '';
        const outputPath = this.replaceFileExtToJs(options.filePath);
        const mapFilePath = `${outputPath}.map`;
        const outputText = result.map ? `${code}\n\n//# sourceMappingURL=${mapFilePath}` : code;
        const outputFiles = [{ outputText, outputPath }];
        if (result.map) {
            outputFiles.push({
                outputText: JSON.stringify(result.map),
                outputPath: mapFilePath,
            });
        }
        return outputFiles;
    }
    /**
     * compile multiple components on the capsules
     */
    build(context) {
        return __awaiter(this, void 0, void 0, function* () {
            const capsules = context.capsuleGraph.seedersCapsules;
            const componentsResults = [];
            const longProcessLogger = this.logger.createLongProcessLogger('compile babel components', capsules.length);
            yield bluebird_1.mapSeries(capsules, (capsule) => __awaiter(this, void 0, void 0, function* () {
                const currentComponentResult = {
                    errors: [],
                    component: capsule.component,
                };
                longProcessLogger.logProgress(capsule.component.id.toString());
                yield this.buildOneCapsule(capsule, currentComponentResult);
                componentsResults.push(Object.assign({}, currentComponentResult));
            }));
            return {
                artifacts: this.getArtifactDefinition(),
                componentsResults,
            };
        });
    }
    createTask(name = 'BabelCompiler') {
        return this.compiler.createTask(name, this);
    }
    buildOneCapsule(capsule, componentResult) {
        return __awaiter(this, void 0, void 0, function* () {
            componentResult.startTime = Date.now();
            const sourceFiles = capsule.component.filesystem.files.map((file) => file.relative);
            yield fs_extra_1.default.ensureDir(path_1.default.join(capsule.path, this.distDir));
            yield Promise.all(sourceFiles.map((filePath) => __awaiter(this, void 0, void 0, function* () {
                var _a;
                let result;
                try {
                    result = yield babel.transformFileAsync(path_1.default.join(capsule.path, filePath), this.options.babelTransformOptions);
                }
                catch (err) {
                    (_a = componentResult.errors) === null || _a === void 0 ? void 0 : _a.push(err);
                }
                if (!result || !result.code) {
                    return;
                }
                const distPath = this.replaceFileExtToJs(filePath);
                const distPathMap = `${distPath}.map`;
                const code = result.code || '';
                const outputText = result.map ? `${code}\n\n//# sourceMappingURL=${distPathMap}` : code;
                capsule.fs.writeFileSync(path_1.default.join(this.distDir, distPath), outputText);
                if (result.map) {
                    capsule.fs.writeFileSync(path_1.default.join(this.distDir, distPathMap), JSON.stringify(result.map));
                }
            })));
            componentResult.endTime = Date.now();
        });
    }
    getArtifactDefinition() {
        return [
            {
                generatedBy: this.id,
                name: this.artifactName,
                globPatterns: this.distGlobPatterns,
            },
        ];
    }
    /**
     * given a source file, return its parallel in the dists. e.g. index.ts => dist/index.js
     */
    getDistPathBySrcPath(srcPath) {
        const fileWithJSExtIfNeeded = this.replaceFileExtToJs(srcPath);
        return path_1.default.join(this.distDir, fileWithJSExtIfNeeded);
    }
    /**
     * whether babel is able to compile the given path
     */
    isFileSupported(filePath) {
        return ((filePath.endsWith('.ts') ||
            filePath.endsWith('.tsx') ||
            filePath.endsWith('.js') ||
            filePath.endsWith('.jsx')) &&
            !filePath.endsWith('.d.ts'));
    }
    displayConfig() {
        return JSON.stringify(this.options.babelTransformOptions || {}, null, 4);
    }
    replaceFileExtToJs(filePath) {
        if (!this.isFileSupported(filePath))
            return filePath;
        const fileExtension = path_1.default.extname(filePath);
        return filePath.replace(new RegExp(`${fileExtension}$`), '.js'); // makes sure it's the last occurrence
    }
}
exports.BabelCompiler = BabelCompiler;
//# sourceMappingURL=babel.compiler.js.map